package com.yeejoin.amos.boot.module.jg.biz.edit.process.biz;

import com.yeejoin.amos.boot.module.jg.api.dto.BizRelationDataDto;
import com.yeejoin.amos.boot.module.jg.api.dto.ChangeDataDto;
import com.yeejoin.amos.boot.module.jg.biz.context.EquipDataProcessStrategyContext;
import com.yeejoin.amos.boot.module.jg.biz.edit.core.IEventPublisher;
import com.yeejoin.amos.boot.module.jg.biz.edit.event.BaseBizDataChangeEvent;
import com.yeejoin.amos.boot.module.jg.biz.edit.process.biz.strategy.IBizDataChangeHandleStrategy;
import com.yeejoin.amos.boot.module.jg.biz.edit.process.equip.strategy.IEquipChangeDataProcessStrategy;
import org.typroject.tyboot.core.foundation.context.RequestContext;

import java.util.List;
import java.util.Map;

public abstract class DefaultBizDataChangeHandler<U extends BaseBizDataChangeEvent> implements IBizDataChangeHandleStrategy {

    private final IEventPublisher<BaseBizDataChangeEvent> eventPublisher;

    protected DefaultBizDataChangeHandler(IEventPublisher<BaseBizDataChangeEvent> eventPublisher) {
        this.eventPublisher = eventPublisher;
    }


    @Override
    public final void doSave(String applyNo, ModelType model, Map<String, Object> changeData) {
        if (beforeCheck(applyNo, model, changeData)) {
            IEquipChangeDataProcessStrategy dataProcessor = EquipDataProcessStrategyContext.getStrategy(model);
            List<ChangeDataDto> allChangeColumns = dataProcessor.handle(changeData, applyNo);
            List<ChangeDataDto> bizEditColumns = bizDataSave(applyNo, model, changeData);
            allChangeColumns.addAll(bizEditColumns);
            // 发送数据变更实践
            publish2OtherBiz(allChangeColumns, applyNo);
        }
    }

    private void publish2OtherBiz(List<ChangeDataDto> allChangeColumns, String applyNo) {
        BizRelationDataDto bizRelationDataDto = new BizRelationDataDto();
        bizRelationDataDto.setBizId(applyNo);
        bizRelationDataDto.setBizType(canHandleBizType());
        bizRelationDataDto.setRecords(getEqs(applyNo));
        eventPublisher.publish(new BaseBizDataChangeEvent(this, bizRelationDataDto, allChangeColumns, RequestContext.cloneRequestContext()));
    }
}
