package com.yeejoin.amos.boot.module.jg.biz.edit.process.biz;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.yeejoin.amos.boot.module.jg.api.dto.ChangeDataDto;
import com.yeejoin.amos.boot.module.jg.api.entity.JgUseRegistration;
import com.yeejoin.amos.boot.module.jg.biz.edit.core.RouterEventPublisher;
import com.yeejoin.amos.boot.module.jg.biz.edit.event.BaseBizDataChangeEvent;
import com.yeejoin.amos.boot.module.jg.biz.service.impl.JgUseRegistrationServiceImpl;
import com.yeejoin.amos.boot.module.jg.biz.service.impl.UseRegisterUpdateService;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class UseRegisterDataChangeHandler extends DefaultBizDataChangeHandler<BaseBizDataChangeEvent> {


    private final UseRegisterUpdateService useRegisterUpdateService;

    protected UseRegisterDataChangeHandler(RouterEventPublisher routerEventPublisher, UseRegisterUpdateService useRegisterUpdateService) {
        super(routerEventPublisher);
        this.useRegisterUpdateService = useRegisterUpdateService;
    }


    @Override
    public String canHandleBizType() {
        return "tzs_jg_use_registration";
    }


    /**
     * 保存
     *
     * @param applyNo    单据编号
     * @param model      single, batch
     * @param changeData 前端不支持分组-目前为平铺数据结构
     */
    @Override
    public List<ChangeDataDto> bizDataSave(String applyNo, ModelType model, Map<String, Object> changeData) {
        List<ChangeDataDto> bizEditColumns = new ArrayList<ChangeDataDto>();
        // 单据信息的保存、与设备相关、证相关字段的更新
        // 1.根据任务状态进行单据数据的处理
        // 1.1 流程中，更新业务相关字段、更新设备
        // 1.2 已完成，更新业务相关字段、证相关、批量方式的设备公共字段信息
        // 选择是台套、还是单位办理的方式，进行分类数据的解析
        // 更新历史的JSON的数据
        this.updateHistoryJson(applyNo);
        return bizEditColumns;
    }

    @Override
    public Boolean beforeCheck(String applyNo, ModelType model, Map<String, Object> changeData) {
        return true;
    }

    @Override
    public Set<String> getEqs(String applyNo) {
        return Collections.emptySet();
    }

    private void updateHistoryJson(String applyNo) {
        useRegisterUpdateService.updateHisData(applyNo);
    }


    @Override
    public Map<String, ?> getDetail(String applyNo, String bizId) {
        Map<String, Object> detail = new HashMap<>();
        JgUseRegistration useRegistration = useRegisterUpdateService.getUseRegistrationService().getOne(new LambdaQueryWrapper<JgUseRegistration>().eq(JgUseRegistration::getApplyNo, applyNo));
        JSONObject lastDetailDataForEdit = useRegisterUpdateService.getLastDetailDataForEdit(useRegistration);
        BeanUtil.copyProperties(lastDetailDataForEdit, detail);
        BeanUtil.copyProperties(useRegistration, detail);
        // 转化 附件 字段
        useRegisterUpdateService.convertStringToJsonObject(detail, JgUseRegistrationServiceImpl.jsonFields);
        return detail;
    }

}
